import functools
from passlib.context import CryptContext

__all__ = ["buildHoneyCredHook", "cryptcontext"]

cryptcontext = CryptContext(schemes=["pbkdf2_sha512","bcrypt", "sha512_crypt", "plaintext"])

def buildHoneyCredHook(creds):
    return functools.partial(testManyCreds, creds)

def testCred(cred, username=None, password=None):
    """
    Test if given credentials matches specified credentials

    If specified credentials doesn't have username or password, it
    will still match on the other.

    """
    cred_username = cred.get("username", None)
    cred_password = cred.get("password", None)

    user_match = True
    if cred_username is not None:
        user_match = (cred_username == username)

    password_match = True
    if cred_password is not None:
        password_match = cryptcontext.verify(password, cred_password)

    return (user_match and password_match)

def testManyCreds(creds, username=None, password=None):
    for c in creds:
        if testCred(c, username, password):
            return True
    return False



import sys

if sys.platform.startswith("linux"):
    from opencanary.modules import CanaryService, FileSystemWatcher
    from opencanary.config import ConfigException

    import os, re, random, shutil, time
    from datetime import datetime

    class SambaLogWatcher(FileSystemWatcher):

        def __init__(self, logFile=None, logger=None):
            self.logger = logger
            FileSystemWatcher.__init__(self, fileName=logFile)

        def handleLines(self, lines=None):
            #samba4 re
            audit_re = re.compile(r'.*smbd_audit: (.*)')

            #samba 3 re
            #audit_re = re.compile(r'.*smbd\[[0-9]+\]: (.*)')
            for line in lines:
                    matches = audit_re.match(line)
                    (user,remoteIP,localIP,remoteName,shareName,
                    localName,smbVer,smbArch,timeStamp,domainName,
                    auditAction,auditStatus,fileName) = matches.group(1).split('|')

                    data = {}
                    data['src_host'] = remoteIP
                    data['src_port'] = '-1'
                    data['dst_host'] = localIP
                    data['dst_port'] = 445
                    data['logtype'] =  self.logger.LOG_SMB_FILE_OPEN
                    data['logdata'] = {'USER':user, 'REMOTENAME': remoteName, 'SHARENAME': shareName,
                                       'LOCALNAME': localName, 'SMBVER': smbVer, 'SMBARCH': smbArch,
                                       'DOMAIN': domainName, 'AUDITACTION': auditAction,
                                       'STATUS':auditStatus, 'FILENAME': fileName}
                    self.logger.log(data)

    class CanarySamba(CanaryService):
        NAME = 'smb'
        def __init__(self,config=None, logger=None):
            CanaryService.__init__(self, config=config, logger=logger)
            self.audit_file = config.getVal('smb.auditfile', default='/var/log/samba-audit.log')
            self.config = config

        def startYourEngines(self, reactor=None):
            #create samba run dir, so testparm doesn't error
            #try:
            #    os.stat('/var/run/samba')
            #except OSError:
            #    os.mkdir('/var/run/samba')

            fs = SambaLogWatcher(logFile=self.audit_file, logger=self.logger)
            fs.start()





import os
import datetime

from opencanary.modules import CanaryService

from base64 import b64decode
import urlparse
from urllib import quote as urlquote
from twisted.application import internet
from twisted.internet.protocol import ServerFactory
from twisted.application.internet import TCPServer
from twisted.internet.protocol import ClientFactory
from twisted.internet import protocol

from twisted.web.http import HTTPClient, Request, HTTPChannel
from twisted.web import http
from twisted.internet import reactor

from jinja2 import Template

PROFILES = {
    "ms-isa" : {
        # p/Microsoft ISA Server Web Proxy/
        # Force HTTP/1.1 reply even when sent HTTP/1.0 (to match the nmap version sig)
        "HTTP1.1_always": True,
        "headers": [
            ("Via", "1.1 localhost"),
            ("Proxy-Authenticate", "Basic"),
            # TODO: more realistict authentication for ISA
            # ("Proxy-Authenticate", "NTLM"),
            # ("Proxy-Authenticate", "Kerberos"),
            # ("Proxy-Authenticate", "Negotiate"),
            ("Pragma", "no-cache"),
            ("Cache-Control", "no-cache"),
        ],
        "status_reason": "Proxy Authentication Required ( The ISA Server requires authorization to fulfill the request. Access to the Web Proxy service is denied.  )"
    },
    "squid" : {
        # p/Squid http proxy/ v/$1/ cpe:/a:squid-cache:squid:$1/
        "banner": 'Squid proxy-caching web server',
        "headers": [
            ("Server", "squid/3.3.8"),
            ("Mime-Version", "1.0"),
            ("Vary", "Accept-Language"),
            ("Via", "1.1 localhost (squid/3.3.8)"),
            ("X-Cache", "MISS from localhost"),
            ("X-Cache-Lookup", "NONE from localhost"), # actually hostname:port
            ("X-Squid-Error", "ERR_CACHE_ACCESS_DENIED 0")
        ],
        "status_reason": "Proxy Authentication Required"
    }
}

class AlertProxyRequest(Request):
    """
    Used by Proxy to implement a simple web proxy.
    """

    FACTORY=None

    def __init__(self, channel, queued):
        Request.__init__(self, channel, queued)

    def logAuth(self):
        auth = self.getHeader("Proxy-Authorization")
        if auth is None:
            return

        factory = AlertProxyRequest.FACTORY

        username, password = "Invalid auth-token submitted", ""
        atype, token  = auth.split(" ")
        if atype == "Basic":
            try:
                username, password = b64decode(token).split(":")
            except:
                pass
        elif atype == "NTLM":
            print b64decode(token).split(":")
            exit(1)
            print "something NTLM"
            return

        logdata = {'USERNAME': username, 'PASSWORD': password}
        factory.log(logdata, transport=self.transport)

    def process(self):
        self.logAuth()

        factory = AlertProxyRequest.FACTORY
        profile = PROFILES[factory.skin]
        content = factory.auth_template.render(
            url=self.uri,
            date=datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S %ZGMT"),
            clientip=self.transport.getPeer().host
        )
        if factory.banner:
            prompt = factory.banner
        else:
            prompt = profile.get("banner","")

        #  for fooling nmap service detection
        if profile.get("HTTP1.1_always", False):
            self.clientproto = "HTTP/1.1"

        # match http-proxy m|^HTTP/1\.[01] \d\d\d .*\r\nServer: [sS]quid/([-.\w+]+)\r\n|s
        self.setResponseCode(407, profile["status_reason"])
        for (name, value) in profile["headers"]:
            self.responseHeaders.addRawHeader(name, value)

        self.responseHeaders.addRawHeader("Content-Type", "text/html")
        self.responseHeaders.addRawHeader("Proxy-Authenticate",
                                           'Basic realm="%s"' % prompt)
        self.responseHeaders.addRawHeader("Content-Length", len(content))

        self.write(content.encode("utf-8"))
        self.finish()

class AlertProxy(HTTPChannel):
    requestFactory = AlertProxyRequest

class HTTPProxyFactory(http.HTTPFactory):
    def buildProtocol(self, addr):
        return AlertProxy()

class HTTPProxy(CanaryService):
    NAME = 'httpproxy'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('httpproxy.port', default=8443))
        self.banner = config.getVal('httpproxy.banner', '').encode('utf8')
        self.skin = config.getVal('httpproxy.skin', default='squid')
        self.skindir = os.path.join(
            HTTPProxy.resource_dir(), 'skin', self.skin)
        self.logtype = logger.LOG_HTTPPROXY_LOGIN_ATTEMPT
        self.listen_addr = config.getVal('device.listen_addr', default='')

        authfilename = os.path.join(self.skindir, 'auth.html')
        try:
            with open(authfilename, 'r') as f:
                self.auth_template = Template(f.read())
        except:
            self.auth_template = Template("")


    def getService(self):
        AlertProxyRequest.FACTORY = self
        f = HTTPProxyFactory()
        return internet.TCPServer(self.port, f, interface=self.listen_addr)



from opencanary.modules import CanaryService

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

class ProtocolError(Exception):
    pass

class GitProtocol(Protocol):
    """
        Implementation of Git-daemon up to request
    """

    def _checkDataLength(self, data):
        try:
            actual_length = len(data)
            indata_length = int(data[0:4],base=16)
            if(actual_length == indata_length):
                return True
            else:
                return False
        except ValueError:
            return False

    def _buildResponseAndSend(self,command):
        project = command[17:17 + command[17:].find('host')]
        request = command[command.find('=')+1:]
        self._logAlert(project,request)
        pre_response = 'ERR no such respository: ' + project
        response_size = '{:04x}'.format(int(len(pre_response) + 4))
        response = response_size + pre_response
        self.transport.write(response + '\n')

    def _logAlert(self, project,request):
        logdata = {"REPO": project[:-1],
                   "HOST": request[:-1]}
        self.factory.log(logdata, transport=self.transport)

    def dataReceived(self, data):
        """
        Recieved data is unbuffered so we buffer it for telnet.
        """
        try:
            git_command = data[4:]
            if(self._checkDataLength(data) and git_command[:15]=='git-upload-pack'):
                self._buildResponseAndSend(git_command)
            else:
                raise ProtocolError()
        except (ProtocolError):
            self.transport.loseConnection()
            return

class CanaryGit(Factory, CanaryService):
    NAME = 'git'
    protocol = GitProtocol

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = config.getVal("git.port", default=9418)
        self.logtype = logger.LOG_GIT_CLONE_REQUEST

    def getService(self):
        return internet.TCPServer(self.port, self)



from opencanary.modules import CanaryService

from zope.interface import implements
from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol

from twisted.application.internet import UDPServer
from twisted.internet.address import IPv4Address

from twisted.internet import protocol

from scapy.all import SNMP
"""
    A log-only SNMP server. It won't respond, but it will log SNMP queries.
"""

class MiniSNMP(DatagramProtocol):
    def datagramReceived(self, data, (host, port)):
        try:
            snmp = SNMP(data)
            community = snmp.community.val
            requests = [x.oid.val for x in snmp.PDU.varbindlist]

            logdata={'REQUESTS': requests, 'COMMUNITY_STRING': community}
            self.transport.getPeer = lambda: IPv4Address('UDP', host, port)
            self.factory.log(logdata=logdata, transport=self.transport)
        except Exception as e:
            print e
            pass


class CanarySNMP(CanaryService):
    NAME = 'SNMP'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('snmp.port', default=161))
        self.logtype = logger.LOG_SNMP_CMD
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        f = MiniSNMP()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)



from opencanary.modules import CanaryService

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

from des import des

import os

RFB_33  = '003.003'
RFB_37  = '003.007'
RFB_38  = '003.008'

#states
PRE_INIT = 1
HANDSHAKE_SEND = 2
SECURITY_SEND = 3
AUTH_SEND = 4
AUTH_OVER = 5

#if one of these is used in the VNC authentication attempt, alert that 
#a common password was tried
COMMON_PASSWORDS=['111111', 'password', '123456', '111111','1234',
                  'administrator','root','passw0rd']

class ProtocolError(Exception):
    pass

class UnsupportedVersion(Exception):
    pass

class VNCProtocol(Protocol):
    """
        Implementation of VNC up to VNC authentication
    """
    def __init__(self, version=RFB_38):
        self.serv_version = version
        self.state = PRE_INIT

    def _send_handshake(self,):
        print 'send handshake'
        self.transport.write('RFB {version}\n'.format(version=self.serv_version))
        self.state = HANDSHAKE_SEND

    def _recv_handshake(self,data=None):
        print 'got handshake'
        if len(data) != 12 or data[:3] != 'RFB':
            raise ProtocolError()
        client_ver = data[4:-1]

        #support single version for now
        if client_ver not in [RFB_33, RFB_37, RFB_38]:
            raise UnsupportedVersion()

        self._send_security(client_ver)

    def _send_security(self,client_ver):
        print 'send security'
        if client_ver == RFB_33:
            self.transport.write('\x00\x00\x00\x02')#specify VNC auth using 4 bytes
            self._send_auth()
        else:
            self.transport.write('\x01\x02')#VNC authentication
            self.state = SECURITY_SEND

    def _recv_security(self,data=None):
        print 'got security'
        if len(data) != 1 and data != '\x02':
            raise ProtocolError()
        self._send_auth()

    def _send_auth(self,):
        print 'send auth'
        self.challenge = os.urandom(16)
        self.transport.write(self.challenge)
        self.state = AUTH_SEND

    def _recv_auth(self,data=None):
        print 'got auth'
        if len(data) != 16:
            raise ProtocolError()
        logdata = {"VNC Server Challenge" : self.challenge.encode('hex'),
                   "VNC Client Response": data.encode('hex')}

        used_password = self._try_decrypt_response(response=data)
        if used_password:
            logdata['VNC Password'] = used_password
        else:
            logdata['VNC Password'] = '<Password was not in the common list>'
        self.factory.log(logdata, transport=self.transport)
        self._send_auth_failed()

    def connectionMade(self):
        if self.state != PRE_INIT:
            raise ProtocolError()
        self._send_handshake()

    def _send_auth_failed(self,):
        self.transport.write('\x00\x00\x00\x01'+#response code
                             '\x00\x00\x00\x16'+#message length
                             'Authentication failure')#Message
        self.state = AUTH_OVER
        raise ProtocolError()

    def _try_decrypt_response(self, response=None):
        #attempt to decrypt each of the common passwords
        #really inefficient, but it means we don't have to rely on
        #a static challenge
        for password in COMMON_PASSWORDS:
            pw = password[:8]#vnc passwords are max 8 chars
            if len(pw) < 8:
                pw+= '\x00'*(8-len(pw))

            #VNC use of DES requires password bits to be mirrored
            pw = ''.join([chr(int('{:08b}'.format(ord(x))[::-1], 2))
                                                       for x in pw])
            desbox = des(pw)
            decrypted_challenge = desbox.decrypt(response)
            if decrypted_challenge == self.challenge:
                return password
        return None

    def dataReceived(self, data):
        """
        Recieved data is unbuffered so we buffer it for telnet.
        """
        try:
            if self.state == HANDSHAKE_SEND:
                self._recv_handshake(data=data)
            elif self.state == SECURITY_SEND:
                self._recv_security(data=data)
            elif self.state == AUTH_SEND:
                self._recv_auth(data=data)
        except (UnsupportedVersion, ProtocolError):
            self.transport.loseConnection()
            return

class CanaryVNC(Factory, CanaryService):
    NAME = 'VNC'
    protocol = VNCProtocol

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config, logger)
        self.port = config.getVal("vnc.port", 5900)
        self.logtype = logger.LOG_VNC

CanaryServiceFactory = CanaryVNC



from opencanary.modules import CanaryService
from opencanary.modules import FileSystemWatcher
import os
import re

class SynLogWatcher(FileSystemWatcher):
    def __init__(self, logger=None, logFile=None):
        self.logger = logger
        #print ('SynLogWatcher started')
        FileSystemWatcher.__init__(self, fileName=logFile)

    def handleLines(self, lines=None):
        for line in lines:
            try:
                if 'canaryfw: ' in line:
                    logtype = self.logger.LOG_PORT_SYN
                    (rubbish, log) = line.split('canaryfw: ')
                elif "canarynmapNULL" in line:
                    logtype = self.logger.LOG_PORT_NMAPNULL
                    (rubbish, log) = line.split('canarynmapNULL: ')
                elif "canarynmapXMAS" in line:
                    logtype = self.logger.LOG_PORT_NMAPXMAS
                    (rubbish, log) = line.split('canarynmapXMAS: ')
                elif "canarynmapFIN" in line:
                    logtype = self.logger.LOG_PORT_NMAPFIN
                    (rubbish, log) = line.split('canarynmapFIN: ')
                elif 'canarynmap: ' in line:
                    logtype = self.logger.LOG_PORT_NMAPOS
                    (rubbish, log) = line.split('canarynmap: ')
                else:
                    continue
            except ValueError:
                continue
            tags = log.split(' ')
            kv = {}
            for tag in tags:
                if tag.find('=') >= 0:
                    (key, val) = tag.split('=')
                else:
                    key = tag
                    val = ''
                kv[key]=val

            try:
                #we've seen empty tags creep in. weed them out.
                kv.pop('')
            except:
                pass

            data = {}
            data['src_host'] = kv.pop('SRC')
            data['src_port'] = kv.pop('SPT')
            data['dst_host'] = kv.pop('DST')
            data['dst_port'] = kv.pop('DPT')
            data['logtype']  = logtype
            data['logdata']  = kv
            self.logger.log(data)

class CanaryPortscan(CanaryService):
    NAME = 'portscan'

    def __init__(self,config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.audit_file = config.getVal('portscan.logfile', default='/var/log/kern.log')
        self.synrate = int(config.getVal('portscan.synrate', default=5))
        self.nmaposrate = config.getVal('portscan.nmaposrate', default='5')
        self.lorate = config.getVal('portscan.lorate', default='3')
        self.listen_addr = config.getVal('device.listen_addr', default='')
        self.config = config

        try:
            self.synrate = int(self.synrate)
        except:
            self.synrate = 5

        try:
            self.nmaposrate = int(self.nmaposrate)
        except:
            self.nmaposrate = 5

        try:
            self.lorate = int(self.lorate)
        except:
            self.lorate = 3

    def startYourEngines(self, reactor=None):
        # Logging rules for loopback interface.
        # This is separate from the canaryfw rule as the canary watchdog was
        # causing console-side noise in the logs.
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp -i lo -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{0}/hour"'.format(self.lorate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp -i lo -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{0}/hour"'.format(self.lorate))

        # Logging rules for canaryfw.
        # We ignore loopback interface traffic as it is taken care of in above rule
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp --syn -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{0}/second" ! -i lo'.format(self.synrate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp --syn -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{0}/second" ! -i lo'.format(self.synrate))

        # os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp {dst} --syn -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{synrate}/second"'
        #             .format(dst=(('--destination '+self.listen_addr) if len(self.listen_addr) else ''),
        #                 synrate=self.synrate))
        # os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp {dst} --syn -j LOG --log-level=warning --log-prefix="canaryfw: " -m limit --limit="{synrate}/second"'
        #             .format(dst=(('--destination '+self.listen_addr) if len(self.listen_addr) else ''),
        #                 synrate=self.synrate))

         # Match the T3 probe of the nmap OS detection based on TCP flags and TCP options string
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp --tcp-flags ALL URG,PSH,SYN,FIN -m u32 --u32 "40=0x03030A01 && 44=0x02040109 && 48=0x080Affff && 52=0xffff0000 && 56=0x00000402" -j LOG --log-level=warning --log-prefix="canarynmap: " -m limit --limit="{0}/second"'.format(self.nmaposrate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp --tcp-flags ALL URG,PSH,SYN,FIN -m u32 --u32 "40=0x03030A01 && 44=0x02040109 && 48=0x080Affff && 52=0xffff0000 && 56=0x00000402" -j LOG --log-level=warning --log-prefix="canarynmap: " -m limit --limit="{0}/second"'.format(self.nmaposrate))

        # Nmap Null Scan
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50000400" -j LOG --log-level=warning --log-prefix="canarynmapNULL: " -m limit --limit="{0}/second"'.format(self.nmaposrate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50000400" -j LOG --log-level=warning --log-prefix="canarynmapNULL: " -m limit --limit="{0}/second"'.format(self.nmaposrate))

        # Nmap Xmas Scan
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50290400" -j LOG --log-level=warning --log-prefix="canarynmapXMAS: " -m limit --limit="{0}/second"'.format(self.nmaposrate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50290400" -j LOG --log-level=warning --log-prefix="canarynmapXMAS: " -m limit --limit="{0}/second"'.format(self.nmaposrate))

        # Nmap Fin Scan
        os.system('sudo /sbin/iptables -t mangle -D PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50010400" -j LOG --log-level=warning --log-prefix="canarynmapFIN: " -m limit --limit="{0}/second"'.format(self.nmaposrate))
        os.system('sudo /sbin/iptables -t mangle -A PREROUTING -p tcp -m u32 --u32 "6&0xFF=0x6 && 0>>22&0x3C@12=0x50010400" -j LOG --log-level=warning --log-prefix="canarynmapFIN: " -m limit --limit="{0}/second"'.format(self.nmaposrate))

        fs = SynLogWatcher(logFile=self.audit_file, logger=self.logger)
        fs.start()

    def configUpdated(self,):
        pass



from opencanary.modules import CanaryService
import socket

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

class ProtocolError(Exception):
    pass

class UnsupportedVersion(Exception):
    pass

class TCPBannerProtocol(Protocol):
    """
        Implementation of TCP Banner module - reply with a text banner
    """
    def __init__(self, factory, banner_id , accept_banner, send_banner,
                 alert_string_enabled ,alert_string, keep_alive_enabled, keep_alive_secret,
                 keep_alive_idle, keep_alive_interval, keep_alive_probes):
        self.factory = factory
        self.banner_id = banner_id
        self.accept_banner = accept_banner
        self.send_banner = send_banner
        self.alert_string_enabled = alert_string_enabled
        self.alert_string = alert_string
        self.keep_alive_enabled = keep_alive_enabled
        # once we send the secret key we disable alerting on keep-alive
        # connections
        self.keep_alive_disable_alerting = False
        self.keep_alive_secret = keep_alive_secret
        self.keep_alive_idle = keep_alive_idle
        self.keep_alive_interval = keep_alive_interval
        self.keep_alive_probes = keep_alive_probes

    def connectionMade(self):
        #We limit the data sent through to 255 chars
        data = str(self.accept_banner)[:255]

        logdata = {'FUNCTION': 'CONNECTION_MADE', 'DATA':data,
                   'BANNER_ID':str(self.banner_id)}

        if self.keep_alive_enabled:
            if hasattr(socket, 'TCP_KEEPIDLE'):
                # overrides value (in seconds) of system-wide ipv4 tcp_keepalive_time
                self.transport.getHandle().setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, self.keep_alive_idle)
            # overrides value (in seconds) of system-wide ipv4 tcp_keepalive_intvl
            self.transport.getHandle().setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, self.keep_alive_interval)
            # overrides value (in seconds) of system-wide ipv4 tcp_keepalive_probes
            self.transport.getHandle().setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, self.keep_alive_probes)
            # set keep alive on socket
            self.transport.setTcpKeepAlive(1)

            self.factory.canaryservice.logtype = self.factory.canaryservice.logger.LOG_TCP_BANNER_KEEP_ALIVE_CONNECTION_MADE
            self.factory.canaryservice.log(logdata, transport=self.transport)

        elif not self.alert_string_enabled:
            #flag says we need to wait for incoming data to include a string
            #so no point in logging anything here

            self.factory.canaryservice.logtype = self.factory.canaryservice.logger.LOG_TCP_BANNER_CONNECTION_MADE
            self.factory.canaryservice.log(logdata, transport=self.transport)

        self.transport.write(self.accept_banner)

    def dataReceived(self, data):
        """
        Recieved data from tcp connection after connection has been made.
        """
        try:
            if self.keep_alive_disable_alerting:
                self.transport.write(self.send_banner)
                return

            #We limit the data sent through to 255 chars
            data = data[:255]

            logdata = {'FUNCTION':'DATA_RECEIVED',
                       'BANNER_ID':str(self.banner_id)}
            try:
                logdata['DATA'] = str(data).rstrip().decode('utf-8').encode('utf-8')
            except UnicodeDecodeError:
                logdata['DATA'] = str(data).rstrip().decode('unicode_escape').encode('utf-8')

            send_log = True

            if self.keep_alive_enabled:
                if self.keep_alive_secret != '' and self.keep_alive_secret in data:
                    self.keep_alive_disable_alerting = True
                    self.factory.canaryservice.logtype = self.factory.canaryservice.logger.LOG_TCP_BANNER_KEEP_ALIVE_SECRET_RECEIVED
                    logdata['SECRET_STRING'] = str(self.keep_alive_secret)
                else:
                    self.factory.canaryservice.logtype = self.factory.canaryservice.logger.LOG_TCP_BANNER_KEEP_ALIVE_DATA_RECEIVED
            else:
                self.factory.canaryservice.logtype = self.factory.canaryservice.logger.LOG_TCP_BANNER_DATA_RECEIVED
                if self.alert_string_enabled:
                    if self.alert_string in data:
                        logdata['ALERT_STRING'] = str(self.alert_string)
                    else:
                        send_log = False

            if send_log:
                self.factory.canaryservice.log(logdata, transport=self.transport)

            self.transport.write(self.send_banner)
        except (UnsupportedVersion, ProtocolError):
            self.transport.loseConnection()
            return

class TCPBannerFactory(Factory):
    def __init__(self, config=None, banner_id=1):
        self.banner_id = str(banner_id)
        self.accept_banner = config.getVal('tcpbanner_' + self.banner_id + '.initbanner','')\
            .encode('utf8').replace('\\n','\n').replace('\\r','\r')
        self.send_banner = config.getVal('tcpbanner_' + self.banner_id + '.datareceivedbanner','')\
            .encode('utf8').replace('\\n','\n').replace('\\r','\r')
        self.alert_string_enabled = config.getVal('tcpbanner_' + self.banner_id + '.alertstring.enabled', False)
        self.alert_string = config.getVal('tcpbanner_' + self.banner_id + '.alertstring','')\
            .encode('utf8').replace('\\n','\n').replace('\\r','\r')
        self.keep_alive_enabled = config.getVal('tcpbanner_' + self.banner_id + '.keep_alive.enabled', False)
        self.keep_alive_secret = config.getVal('tcpbanner_' + self.banner_id + '.keep_alive_secret', '')\
            .encode('utf8').replace('\\n','\n').replace('\\r','\r')
        # the defaults for the tcp keep alive values add up to 1 hour keep alive
        # 300 for the first probe + 300 * 11 for the interval probes = 3600
        self.keep_alive_idle = config.getVal('tcpbanner_' + self.banner_id + '.keep_alive_idle', 300)
        self.keep_alive_interval = config.getVal('tcpbanner_' + self.banner_id + '.keep_alive_interval', 300)
        self.keep_alive_probes = config.getVal('tcpbanner_' + self.banner_id + '.keep_alive_probes', 11)

    def buildProtocol(self, addr):
        return TCPBannerProtocol(self,self.banner_id ,self.accept_banner,
                                 self.send_banner,self.alert_string_enabled,
                                 self.alert_string, self.keep_alive_enabled,
                                 self.keep_alive_secret, self.keep_alive_idle,
                                 self.keep_alive_interval, self.keep_alive_probes)


class CanaryTCPBanner(CanaryService):
    NAME = 'tcpbanner'
    MAX_TCP_BANNERS = 10

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)

    def getService(self):
        services = []
        for i in range(1,self.config.getVal('tcpbanner.maxnum', default=self.MAX_TCP_BANNERS)+1):
            if self.config.getVal('tcpbanner_'+str(i)+'.enabled', False):
                factory = TCPBannerFactory(config=self.config, banner_id=i)
                factory.canaryservice = self
                port = self.config.getVal('tcpbanner_'+str(i)+'.port',default=8000+i)
                services.append(internet.TCPServer(port, factory))
        return services



from opencanary.modules import CanaryService

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

class Example1Protocol(Protocol):
    """
    Example Telnet Protocol

    $ telnet localhost 8025
    Trying 127.0.0.1...
    Connected to localhost.
    Escape character is '^]'.
    password:
    password:
    password:
    % Bad passwords
    Connection closed by foreign host.

    Nmap's version detection is convinced:

    $ nmap -sV 127.0.0.1 -p 8025
    Starting Nmap 6.47 ( http://nmap.org ) at 2015-07-24 11:40 SAST
    Nmap scan report for localhost (127.0.0.1)
    Host is up (0.000079s latency).
    PORT     STATE SERVICE VERSION
    8025/tcp open  telnet  D-Link ADSL router telnetd
    Service Info: Device: router
    """
    def __init__(self):
        self.prompts = 0
        self.buffer = ""

    def connectionMade(self):
        self.transport.write("\xff\xfb\x03\xff\xfb\x01password: ")
        self.prompts += 1

    def dataReceived(self, data):
        """
        Recieved data is unbuffered so we buffer it for telnet.
        """
        self.buffer += data
        print "Recieved data: ", repr(data)

        # Discard inital telnet client control chars
        i = self.buffer.find("\x01")
        if i >= 0:
            self.buffer = self.buffer[i+1:]
            return

        if self.buffer.find("\x00") >= 0:
            password = self.buffer.strip("\r\n\x00")
            logdata = {"PASSWORD" : password}
            self.factory.log(logdata, transport=self.transport)
            self.buffer = ""

            if self.prompts < 3:
                self.transport.write("\r\npassword: ")
                self.prompts += 1
            else:
                self.transport.write("\r\n% Bad passwords\r\n")
                self.transport.loseConnection()

class CanaryExample1(Factory, CanaryService):
    NAME = 'example1'
    protocol = Example1Protocol

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config, logger)
        self.port = config.getVal("example1.port", 8025)
        self.logtype = logger.LOG_BASE_EXAMPLE

CanaryServiceFactory = CanaryExample1



from opencanary.modules import CanaryService

from twisted.application import internet
from twisted.web.server import Site, GzipEncoderFactory
from twisted.web.resource import Resource, EncodingResourceWrapper, ForbiddenResource
from twisted.web.util import Redirect
from twisted.web import static

import os
import re


class Error(Resource):
    isLeaf = True

    def __init__(self, factory, error_code="404"):
        self.factory = factory
        self.skin = self.factory.skin
        self.skindir = self.factory.skindir
        self.error_code = error_code

        if not os.path.isdir(self.skindir):
            raise Exception(
                "Directory %s for http skin, %s, does not exist." %
                                          (self.skindir,self.skin))

        with open(os.path.join(self.skindir, error_code+".html")) as f:
            self.error_contents = f.read()

        Resource.__init__(self)

    def err_page(self, request):
        path = request.path\
                .replace('<', '&lt;')\
                .replace('>', '&gt;')
        return self.error_contents\
                .replace('[[URL]]', path)\
                .replace('[[BANNER]]', self.factory.banner)

    def render(self, request):
        request.setHeader('Server', self.factory.banner)
        request.setResponseCode(int(self.error_code))
        return Resource.render(self, request)

    def render_GET(self, request):
        return self.err_page(request)

    def render_POST(self, request):
        return self.err_page(request)

class BasicLogin(Resource):
    isLeaf = True

    def __init__(self, factory):
        self.factory = factory
        self.skin = self.factory.skin
        self.skindir = self.factory.skindir

        if not os.path.isdir(self.skindir):
            raise Exception(
                "Directory %s for http skin, %s, does not exist." 
                                       % (self.skindir, self.skin))

        with open(os.path.join(self.skindir, "index.html")) as f:
            text = f.read()

        p = re.compile(r"<!--STARTERR-->.*<!--ENDERR-->", re.DOTALL)
        self.login = re.sub(p, "", text)
        self.err = re.sub(r"<!--STARTERR-->|<!--ENDERR-->", "", text)
        Resource.__init__(self)

    def render(self, request):
        request.setHeader('Server', self.factory.banner)
        return Resource.render(self, request)

    def render_GET(self, request, loginFailed=False):
        if not loginFailed:
            us = request.transport.getHost()
            peer = request.transport.getPeer()
            useragent = request.getHeader('user-agent')
            if not useragent:
                useragent = '<not supplied>'

            logdata = {
                'SKIN': self.skin,
                'HOSTNAME': request.getRequestHostname(),
                'PATH': request.path,
                'USERAGENT': useragent
            }

            logtype = self.factory.logger.LOG_HTTP_GET
            self.factory.log(logdata, transport=request.transport, logtype=logtype)

        return self.login

    def render_POST(self, request):
        try:
            username = request.args['username'][0]
        except KeyError, IndexError:
            username = '<not supplied>'
        try:
            password = request.args['password'][0]
        except KeyError, IndexError:
            password = '<not supplied>'
        useragent = request.getHeader('user-agent')
        if not useragent:
            useragent = '<not supplied>'

        logdata = {
            'USERNAME': username,
            'PASSWORD': password,
            'SKIN': self.skin,
            'HOSTNAME': request.getRequestHostname(),
            'PATH': request.path,
            'USERAGENT': useragent
        }
        logtype = self.factory.logger.LOG_HTTP_POST_LOGIN_ATTEMPT
        self.factory.log(logdata, transport=request.transport, logtype=logtype)

        return self.err


class RedirectCustomHeaders(Redirect):

    def __init__(self, request, factory):
        Redirect.__init__(self, request)
        self.factory = factory

    def render(self, request):
        request.setHeader('Server', self.factory.banner)
        return Redirect.render(self, request)


class StaticNoDirListing(static.File):
    """Web resource that serves static directory tree.

    Directory listing is not allowed, and custom headers are set.

    """

    # Banner is declared statically because Twisted creates several
    # instances of this object, where we don't control the arguments
    # for the initialisation.
    BANNER = None
    FORBIDDEN = None

    def createErrorPages(self, factory):
        self.childNotFound = Error(factory, error_code="404")
        StaticNoDirListing.FORBIDDEN = Error(factory, error_code="403")

    def directoryListing(self):
        return StaticNoDirListing.FORBIDDEN

    def getChild(self, name, request):
        request.setHeader('Server', StaticNoDirListing.BANNER)
        return static.File.getChild(self, name, request)


class CanaryHTTP(CanaryService):
    NAME = 'http'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.skin = config.getVal('http.skin', default='basicLogin')
        self.skindir = os.path.join(
            CanaryHTTP.resource_dir(), "skin", self.skin)
        self.staticdir = os.path.join(self.skindir, "static")
        self.port = int(config.getVal('http.port', default=80))
        ubanner = config.getVal('http.banner', default="Apache/2.2.22 (Ubuntu)")
        self.banner = ubanner.encode('utf8')
        StaticNoDirListing.BANNER = self.banner
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        page = BasicLogin(factory=self)
        root = StaticNoDirListing(self.staticdir)
        root.createErrorPages(self)
        root.putChild("", RedirectCustomHeaders("/index.html", factory=self))
        root.putChild("index.html", page)
        wrapped = EncodingResourceWrapper(root, [GzipEncoderFactory()])
        site = Site(wrapped)
        return internet.TCPServer(self.port, site, interface=self.listen_addr)



from twisted.internet import reactor
from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet
from rdpy.protocol.rdp.rdp import RDPServerObserver
from rdpy.protocol.rdp.rdp import ServerFactory
from rdpy.core import rss
from rdpy.core.scancode import scancodeToChar
from opencanary.modules import CanaryService

class RDPObserver(RDPServerObserver):
    def __init__(self, factory, controller, rssFile):
        RDPServerObserver.__init__(self, controller)
        self.factory = factory
        self.buffer = ""

    def onReady(self):
        domain, username, password = self._controller.getCredentials()
        hostname = self._controller.getHostname()
        logdata = {
            "DOMAIN": domain,
            "USERNAME": username,
            "PASSWORD": password,
            "HOSTNAME": hostname
        }
        transport = self._controller.getProtocol().transport
        us = transport.getHost()
        peer = transport.getPeer()
        self.transportlog = {
            'src_host' : peer.host,
            'src_port' : peer.port,
            'dst_host' : us.host,
            'dst_port' : us.port
        }
        self.factory.log(logdata, **self.transportlog)
        self.doEvent(0)

    def onClose(self):
        if getattr(self, "transportlog", None):
            logdata = {"INPUT" : self.buffer}
            self.factory.log(logdata, **self.transportlog)

    def onKeyEventScancode(self, code, isPressed, isExtended):
        self.buffer += scancodeToChar(code)

    def onKeyEventUnicode(self, code, isPressed):
        pass

    def onPointerEvent(self, x, y, button, isPressed):
        pass

    def doEvent(self, i):
        if i >= len(self.factory.rss) - 1:
            return

        e = self.factory.rss[i]

        # TODO: handle other events (eg. the screen resize)
        if e.type.value == rss.EventType.UPDATE:
            self._controller.sendUpdate(
                e.event.destLeft.value,
                e.event.destTop.value,
                e.event.destRight.value,
                e.event.destBottom.value,
                e.event.width.value, e.event.height.value,
                e.event.bpp.value,
                e.event.format.value == rss.UpdateFormat.BMP,
                e.event.data.value)

        t = self.factory.rss[i+1].timestamp.value
        reactor.callLater(float(t) / 1000.0, self.doEvent, i + 1)

class CanaryRDP(ServerFactory, CanaryService):
    NAME = 'rdp'

    def __init__(self, config=None, logger=None):
        ServerFactory.__init__(self, 16, None, None)
        CanaryService.__init__(self, config, logger)

        self.rssFile = self.resource_filename("login.rss")
        reader = rss.createReader(self.rssFile)
        self.rss = []
        while True:
            e = reader.nextEvent()
            if e:
                self.rss.append(e)
            else:
                break

        self.port = config.getVal("rdp.port", 3389)
        self.logtype = logger.LOG_RDP

    def buildObserver(self, controller, addr):
        return RDPObserver(self, controller, self.rssFile)

CanaryServiceFactory = CanaryRDP



import os, sys, json, copy, socket, itertools, string, subprocess
from os.path import expanduser
from pkg_resources import resource_filename

SAMPLE_SETTINGS = resource_filename(__name__, 'data/settings.json')
SETTINGS = 'opencanary.conf'

def byteify(input):
    if isinstance(input, dict):
        return {byteify(key): byteify(value)
                for key, value in input.iteritems()}
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif isinstance(input, unicode):
        return input.encode('utf-8')
    else:
        return input

class Config:
    def __init__(self, configfile=SETTINGS):
        self.__config = None
        self.__configfile = configfile

        files = [configfile, "%s/.%s" % (expanduser("~"), configfile), "/etc/opencanaryd/%s"%configfile]
        print "** We hope you enjoy using OpenCanary. For more open source Canary goodness, head over to canarytokens.org. **"
        for fname in files:
            try:
                with open(fname, "r") as f:
                    print "[-] Using config file: %s" % fname
                    self.__config = json.load(f)
                    self.__config = byteify(self.__config)
                return
            except IOError as e:
                print "[-] Failed to open %s for reading (%s)" % (fname, e)
            except ValueError as e:
                print "[-] Failed to decode json from %s (%s)" % (fname, e)
                subprocess.call("cp -r %s /var/tmp/config-err-$(date +%%s)" % fname, shell=True)
            except Exception as e:
                print "[-] An error occured loading %s (%s)" % (fname, e)

    def moduleEnabled(self, module_name):
        k = "%s.enabled" % module_name.lower()
        if k in self.__config:
            return bool(self.__config[k])
        return False

    def getVal(self, key, default=None):
        # throw exception to caller
        try:
            return self.__config[key]
        except KeyError as e:
            if default is not None:
                return default
            raise e

    def setValues(self, params):
        """Set all the valid values in params and return a list of errors for invalid"""

        # silently ensure that node_id and mac are not modified via web
        sacred = ["device.node_id", "device.mac"]
        for k in sacred:
            if k in params:
                del params[k]

        # if dhcp is enabled, ignore the static ip settings
        if params.get("device.dhcp.enabled", False):
            static = ["device.ip_address", "device.netmask",
                      "device.gw", "device.dns1", "device.dns2"]
            for k in static:
                if k in params:
                    del params[k]

        # for each section, if disabled, delete ignore section's settings
        disabled_modules = tuple(filter(lambda m: not params.get("%s.enabled" % m, False), ["ftp", "ssh", "smb", "http"]))
        for k in params.keys():
            if not k.endswith("enabled") and k.startswith(disabled_modules):
                del params[k]
                continue

        # test options indpenedently for validity
        errors = []
        for key,value in params.iteritems():
            try:
                self.valid(key,value)
            except ConfigException as e:
                errors.append(e)

        # Test that no ports overlap
        ports = {k: v for k, v in self.__config.iteritems() if k.endswith(".port")}
        newports = {k: v for k, v in params.iteritems() if k.endswith(".port")}
        ports.update(newports)
        ports = [(port,setting) for setting, port in ports.iteritems()]
        ports.sort()

        for port, settings in itertools.groupby(ports, lambda x: x[0]):
            settings = list(settings)
            if len(settings) > 1:
                services = ", ".join([s[1].split(".")[0] for s in settings])
                errmsg = "More than one service uses this port (%s)" % services
                for (port, setting) in settings:
                    errors.append(ConfigException(setting, errmsg))

        # Delete invalid settings for which an error is reported
        for err in errors:
            if err.key in params:
                del params[err.key]

        # Update current settings
        self.__config.update(params)
        return errors

    def setVal(self, key, val):
        """Set value only if valid otherwise throw exception"""
        errs = self.setValues({key: val})

        # sucessful update
        if not errs:
            return

        # raise first error reported on the update key
        for e in errs:
            if e.key == key:
                raise e

    def valid(self, key, val):
        """
        Test an the validity of an invidual setting
        Raise config error message on failure.
        TODO: delegate module tests to appropriate module
        """

        if key.endswith(".enabled"):
            if not ((val is True) or (val is False)):
                raise ConfigException(key, "Boolean setting is not True or False (%s)" % val)

        if key.endswith(".port"):
            if (not isinstance(val,int)) or val < 1 or val > 65535:
                raise ConfigException(key, "Invalid port number (%s)" % val)

        # Max length of SSH version string is 255 chars including trailing CR and LF
        # https://tools.ietf.org/html/rfc4253
        if key == "ssh.version" and len(val) > 253:
            raise ConfigException(key, "SSH version string too long (%s..)" % val[:5])

        if key == "smb.filelist":
            extensions = ["PDF", "DOC", "DOCX"]
            for f in val:
                if "name" not in f:
                    raise ConfigException(key, "No filename specified for %s" % f)
                if "type" not in f:
                    raise ConfigException(key, "No filetype specified for %s" % f)
                if not f["name"]:
                    raise ConfigException(key, "Filename cannot be empty")
                if not f["type"]:
                    raise ConfigException(key, "File type cannot be empty")
                if f["type"] not in extensions:
                    raise ConfigException(key, "Extension %s is not supported" % f["type"])

        if key == "device.name":
            allowed_chars = string.ascii_letters + string.digits + "+-#_"

            if len(val) > 100:
                raise ConfigException(key, "Name cannot be longer than 100 characters")
            elif len(val) < 1:
                raise ConfigException(key, "Name ought to be at least one character")
            elif any(map(lambda x: x not in allowed_chars, val)):
                raise ConfigException(key, "Please use only characters, digits, any of the following: + - # _")

        if key == "device.desc":
            allowed_chars = string.ascii_letters + string.digits + "+-#_ "
            if len(val) > 100:
                raise ConfigException(key, "Name cannot be longer than 100 characters")
            elif len(val) < 1:
                raise ConfigException(key, "Name ought to be at least one character")
            elif any(map(lambda x: x not in allowed_chars, val)):
                raise ConfigException(key, "Please use only characters, digits, spaces and any of the following: + - # _")

        return True

    def saveSettings(self):
        """Backup config file to older version and save to new file"""
        try:
            cfg = self.__configfile
            if os.path.isfile(cfg):
                os.rename(cfg, cfg + ".bak")

            with open(cfg, "w") as f:
                json.dump(self.__config, f, sort_keys=True, indent=4, separators=(',', ': '))

        except Exception, e:
            print "[-] Failed to save config file %s" % e
            raise ConfigException("config", "%s" % e)


    def __repr__(self):
        return self.__config.__repr__()

    def __str__(self):
        return self.__config.__str__()

    def toDict(self):
        """ Return all settings as a dict """
        return self.__config

    def toJSON(self):
        """
        JSON representation of config
        """
        return json.dumps(self.__config, sort_keys=True, indent=4, separators=(',', ': '))


class ConfigException(Exception):
    """Exception raised on invalid config value"""

    def __init__(self, key, msg):
        self.key = key
        self.msg = msg

    def __str__(self):
        return "%s: %s" % (self.key, self.msg)

    def __repr__(self):
        return "<%s %s (%s)>" % (self.__class__.__name__, self.key, self.msg)

config = Config()



from opencanary.modules import CanaryService

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

class Example0Protocol(Protocol):
    """
     Example (Fictional) Protocol
    
    $ nc localhost 8007
    Welcome!
    password: wrong0
    password: wrong1
    password: wrong2
    Bad passwords
    $
    """

    def __init__(self):
        self.prompts = 0

    def connectionMade(self):
        self.transport.write("Welcome!\r\npassword: ")
        self.prompts += 1

    def dataReceived(self, data):
        """
        Careful, data recieved here is unbuffered. See example1
        for how this can be better handled.
        """
        password = data.strip("\r\n")
        logdata = {"PASSWORD" : password}
        self.factory.log(logdata, transport=self.transport)
        
        if self.prompts < 3:
            self.transport.write("\r\npassword: ")
            self.prompts += 1
        else:
            self.transport.write("\r\nBad passwords\r\n")
            self.transport.loseConnection()
                
class CanaryExample0(Factory, CanaryService):
    NAME = 'example0'
    protocol = Example0Protocol

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config, logger)
        self.port = 8007
        self.logtype = logger.LOG_BASE_EXAMPLE

CanaryServiceFactory = CanaryExample0



from opencanary.modules import CanaryService

from zope.interface import implements
from twisted.conch.telnet import TelnetTransport, AuthenticatingTelnetProtocol
from twisted.application import internet
from twisted.internet.protocol import ServerFactory
from twisted.application.internet import TCPServer

from twisted.internet import protocol
from twisted.cred import portal
from twisted.cred import credentials
from twisted.conch.telnet import AuthenticatingTelnetProtocol
from twisted.conch.telnet import ITelnetProtocol
from twisted.conch.telnet import TelnetTransport
from twisted.conch.telnet import ECHO

class Realm:
    implements(portal.IRealm)
    
    def requestAvatar(self, avatarId, mind, *interfaces):
        if ITelnetProtocol in interfaces:
            av = MyTelnet()
            av.state = 'Command'
            return ITelnetProtocol, av, lambda:None
        raise NotImplementedError("Not supported by this realm")

class AlertAuthTelnetProtocol(AuthenticatingTelnetProtocol):
    def connectionMade(self):
        # p/Cisco telnetd/ d/router/ o/IOS/ cpe:/a:cisco:telnet/ cpe:/o:cisco:ios/a
        # NB _write() is for raw data and write() handles telnet special bytes
        self.transport._write("\xff\xfb\x01\xff\xfb\x03\xff\xfb\0\xff\xfd\0\xff\xfd\x1f\r\n")
        self.transport.write(self.factory.banner)
        self.transport._write("User Access Verification\r\n\r\nUsername: ")

    def telnet_Password(self, line):
        # Body of this method copied from
        # twisted.conch.telnet
        username, password = self.username, line
        del self.username
        def login(ignored):
            creds = credentials.UsernamePassword(username, password)
            d = self.portal.login(creds, None, ITelnetProtocol)
            d.addCallback(self._cbLogin)
            d.addErrback(self._ebLogin)
        self.transport.wont(ECHO).addCallback(login)

        logdata = {'USERNAME': username, 'PASSWORD': password}
        self.factory.canaryservice.log(logdata, transport=self.transport)
        return 'Discard'

class Telnet(CanaryService):
    NAME = 'telnet'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('telnet.port', default=8023))
        self.banner = config.getVal('telnet.banner', '').encode('utf8')
        self.logtype = logger.LOG_TELNET_LOGIN_ATTEMPT
        self.listen_addr = config.getVal('device.listen_addr', default='')

        if self.banner:
            self.banner += "\n"

    def getService(self):
        r = Realm()
        p = portal.Portal(r)
        f = protocol.ServerFactory()
        f.canaryservice = self
        f.logger = self.logger
        f.banner = self.banner
        f.protocol = lambda: TelnetTransport(AlertAuthTelnetProtocol, p)
        return internet.TCPServer(self.port, f, interface=self.listen_addr)



import simplejson as json
import logging.config
import socket
import sys

from datetime import datetime
from logging.handlers import SocketHandler
from twisted.internet import reactor
import requests

class Singleton(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

def getLogger(config):
    try:
        d = config.getVal('logger')
    except Exception as e:
        print >> sys.stderr, "Error: config does not have 'logger' section"
        exit(1)

    classname = d.get('class', None)
    if classname is None:
        print >> sys.stderr, "Logger section is missing the class key."
        exit(1)

    LoggerClass = globals().get(classname, None)
    if LoggerClass is None:
        print >> sys.stderr, "Logger class (%s) is not defined." % classname
        exit(1)

    kwargs = d.get('kwargs', None)
    if kwargs is None:
        print >> sys.stderr, "Logger section is missing the kwargs key."
        exit(1)

    try:
        logger = LoggerClass(config, **kwargs)
    except Exception as e:
        print >> sys.stderr, "An error occured initialising the logger class"
        print e
        exit(1)

    return logger

class LoggerBase(object):
    LOG_BASE_BOOT                               = 1000
    LOG_BASE_MSG                                = 1001
    LOG_BASE_DEBUG                              = 1002
    LOG_BASE_ERROR                              = 1003
    LOG_BASE_PING                               = 1004
    LOG_BASE_CONFIG_SAVE                        = 1005
    LOG_BASE_EXAMPLE                            = 1006
    LOG_FTP_LOGIN_ATTEMPT                       = 2000
    LOG_HTTP_GET                                = 3000
    LOG_HTTP_POST_LOGIN_ATTEMPT                 = 3001
    LOG_SSH_NEW_CONNECTION                      = 4000
    LOG_SSH_REMOTE_VERSION_SENT                 = 4001
    LOG_SSH_LOGIN_ATTEMPT                       = 4002
    LOG_SMB_FILE_OPEN                           = 5000
    LOG_PORT_SYN                                = 5001
    LOG_PORT_NMAPOS                             = 5002
    LOG_PORT_NMAPNULL                           = 5003
    LOG_PORT_NMAPXMAS                           = 5004
    LOG_PORT_NMAPFIN                            = 5005
    LOG_TELNET_LOGIN_ATTEMPT                    = 6001
    LOG_HTTPPROXY_LOGIN_ATTEMPT                 = 7001
    LOG_MYSQL_LOGIN_ATTEMPT                     = 8001
    LOG_MSSQL_LOGIN_SQLAUTH                     = 9001
    LOG_MSSQL_LOGIN_WINAUTH                     = 9002
    LOG_TFTP                                    = 10001
    LOG_NTP_MONLIST                             = 11001
    LOG_VNC                                     = 12001
    LOG_SNMP_CMD                                = 13001
    LOG_RDP                                     = 14001
    LOG_SIP_REQUEST                             = 15001
    LOG_GIT_CLONE_REQUEST                       = 16001
    LOG_REDIS_COMMAND                           = 17001
    LOG_TCP_BANNER_CONNECTION_MADE              = 18001
    LOG_TCP_BANNER_KEEP_ALIVE_CONNECTION_MADE   = 18002
    LOG_TCP_BANNER_KEEP_ALIVE_SECRET_RECEIVED   = 18003
    LOG_TCP_BANNER_KEEP_ALIVE_DATA_RECEIVED     = 18004
    LOG_TCP_BANNER_DATA_RECEIVED                = 18005
    LOG_USER_0                                  = 99000
    LOG_USER_1                                  = 99001
    LOG_USER_2                                  = 99002
    LOG_USER_3                                  = 99003
    LOG_USER_4                                  = 99004
    LOG_USER_5                                  = 99005
    LOG_USER_6                                  = 99006
    LOG_USER_7                                  = 99007
    LOG_USER_8                                  = 99008
    LOG_USER_9                                  = 99009

    def sanitizeLog(self, logdata):
        logdata['node_id'] = self.node_id
        logdata['local_time'] = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S.%f")
        if not logdata.has_key('src_host'):
            logdata['src_host'] = ''
        if not logdata.has_key('src_port'):
            logdata['src_port'] = -1
        if not logdata.has_key('dst_host'):
            logdata['dst_host'] = ''
        if not logdata.has_key('dst_port'):
            logdata['dst_port'] = -1
        if not logdata.has_key('logtype'):
            logdata['logtype'] = self.LOG_BASE_MSG
        if not logdata.has_key('logdata'):
            logdata['logdata'] = {}
        return logdata

class PyLogger(LoggerBase):
    """
    Generic python logging
    """
    __metaclass__ = Singleton

    def __init__(self, config, handlers, formatters={}):
        self.node_id = config.getVal('device.node_id')

        # Build config dict to initialise
        # Ensure all handlers don't drop logs based on severity level
        for h in handlers:
            handlers[h]["level"] = "NOTSET"

        logconfig = {
            "version": 1,
            "formatters" : formatters,
            "handlers": handlers,
            # initialise all defined logger handlers
            "loggers": {
                self.node_id : {
                    "handlers": handlers.keys()
                }
            }
        }

        try:
            logging.config.dictConfig(logconfig)
        except Exception as e:
            print >> sys.stderr, "Invalid logging config"
            print type(e)
            print e
            exit(1)

        self.logger = logging.getLogger(self.node_id)

    def error(self, data):
        data['local_time'] = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S.%f")
        msg = '[ERR] %r' % json.dumps(data, sort_keys=True)
        print >> sys.stderr, msg
        self.logger.warn(msg)

    def log(self, logdata, retry=True):
        logdata = self.sanitizeLog(logdata)
        self.logger.warn(json.dumps(logdata, sort_keys=True))


class SocketJSONHandler(SocketHandler):
    """Emits JSON messages over TCP delimited by newlines ('\n')"""

    def makeSocket(self, timeout=1):
        s = SocketHandler.makeSocket(self,timeout)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
        return s

    def __init__(self, *args, **kwargs):
        SocketHandler.__init__(self, *args, **kwargs)
        self.retryStart = 0
        self.retryMax = 0
        self.retryFactor = 0

    def send(self, s, attempt=0):
        if attempt >= 10:
            print "Dropping log message due to too many failed sends"
            return

        if self.sock is None:
            self.createSocket()

        if self.sock:
            try:
                # TODO: Weirdly, one the other ends drops the
                # connection for the next msg, sendall still reports
                # successful write on a disconnected socket but then
                # on subsequent writes it fails correctly.
                self.sock.sendall(s)
                return
            except socket.error:
                self.sock.close()
                self.sock = None

        # Here, we've failed to send s so retry
        reactor.callLater(1.5, lambda x: self.send(s, attempt + 1), None)

    def makePickle(self, record):
        return record.getMessage() + "\n"


class HpfeedsHandler(logging.Handler):
    def __init__(self,host,port,ident, secret,channels):
        logging.Handler.__init__(self)
        self.host=str(host)
        self.port=int(port)
        self.ident=str(ident)
        self.secret=str(secret)
        self.channels=map(str,channels)
        hpc=hpfeeds.new(self.host, self.port, self.ident, self.secret)
        hpc.subscribe(channels)
        self.hpc=hpc

    def emit(self, record):
        try:
            msg = self.format(record)
            self.hpc.publish(self.channels,msg)
        except:
            print "Error on publishing to server"

class SlackHandler(logging.Handler):
    def __init__(self,webhook_url):
        logging.Handler.__init__(self)
        self.webhook_url=webhook_url

    def generate_msg(self, alert):
        msg = {}
        msg['pretext'] = "OpenCanary Alert"
        data=json.loads(alert.msg)
        msg['fields']=[]
        for k,v in data.items():
            msg['fields'].append({'title':k, 'value':json.dumps(v) if type(v) is dict else v})
        return {'attachments':[msg]}

    def emit(self, record):
        data = self.generate_msg(record)
        response = requests.post(
            self.webhook_url, json=data
            )
        if response.status_code != 200:
            print ("Error %s sending Slack message, the response was:\n%s" % (response.status_code, response.text))



from opencanary.modules import CanaryService

from twisted.application import internet
from twisted.protocols.ftp import FTPFactory, FTPRealm, FTP, \
                            USR_LOGGED_IN_PROCEED, GUEST_LOGGED_IN_PROCEED, IFTPShell, \
                            AuthorizationError
from twisted.cred.portal import Portal
from zope.interface import implements
from twisted.cred.checkers import ICredentialsChecker
from twisted.python import failure
from twisted.cred import error as cred_error, credentials

FTP_PATH = "/briar/data/ftp"

class DenyAllAccess:
    implements(ICredentialsChecker)

    credentialInterfaces = (credentials.IAnonymous, credentials.IUsernamePassword)

    def requestAvatarId(self, credentials):
        return failure.Failure(cred_error.UnauthorizedLogin())

class LoggingFTP(FTP):
    #ripped from main FTP class, overridden to extract connection info
    def ftp_PASS(self, password):
        """
        Second part of login.  Get the password the peer wants to
        authenticate with.
        """
        if self.factory.allowAnonymous and self._user == self.factory.userAnonymous:
            # anonymous login
            creds = credentials.Anonymous()
            reply = GUEST_LOGGED_IN_PROCEED
        else:
            # user login
            creds = credentials.UsernamePassword(self._user, password)
            reply = USR_LOGGED_IN_PROCEED

        logdata = {'USERNAME': self._user, 'PASSWORD': password}
        self.factory.canaryservice.log(logdata, transport=self.transport)

        del self._user

        def _cbLogin((interface, avatar, logout)):
            assert interface is IFTPShell, "The realm is busted, jerk."
            self.shell = avatar
            self.logout = logout
            self.workingDirectory = []
            self.state = self.AUTHED
            return reply

        def _ebLogin(failure):
            failure.trap(cred_error.UnauthorizedLogin, cred_error.UnhandledCredentials)
            self.state = self.UNAUTH
            raise AuthorizationError

        d = self.portal.login(creds, None, IFTPShell)
        d.addCallbacks(_cbLogin, _ebLogin)
        return d

class CanaryFTP(CanaryService):
    NAME = 'ftp'

    def __init__(self,config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)

        self.banner = config.getVal('ftp.banner', default='FTP Ready.').encode('utf8')
        self.port = config.getVal('ftp.port', default=21)
        # find a place to check that logtype is initialised
        # find a place to check that factory has service attached
        self.logtype = logger.LOG_FTP_LOGIN_ATTEMPT
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        p = Portal(FTPRealm(FTP_PATH), [DenyAllAccess()])
        f = FTPFactory(p)
        f.protocol = LoggingFTP
        f.welcomeMessage = self.banner
        f.canaryservice = self
        return internet.TCPServer(self.port, f, interface=self.listen_addr)



from opencanary.modules import CanaryService

import twisted
from twisted.cred import portal, checkers, credentials, error
from twisted.conch import error, avatar, interfaces as conchinterfaces
from twisted.conch.checkers import SSHPublicKeyDatabase
from twisted.conch.ssh import factory, userauth, connection, keys, session, transport
from twisted.conch.openssh_compat import primes
from twisted.conch.ssh.common import MP
from twisted.internet import reactor, protocol, defer
from twisted. application import internet

from zope.interface import implements
import sys, os, time
import base64, struct

SSH_PATH="/var/tmp"

#pulled from Kippo
from twisted.conch.ssh.common import NS, getNS
class HoneyPotSSHUserAuthServer(userauth.SSHUserAuthServer):
    def serviceStarted(self):
        userauth.SSHUserAuthServer.serviceStarted(self)

        us = self.transport.getHost()
        peer = self.transport.getPeer()

        logdata = {'LOCALVERSION': self.transport.ourVersionString, 'REMOTEVERSION': self.transport.otherVersionString}
        logtype = self.transport.factory.canaryservice.logger.LOG_SSH_REMOTE_VERSION_SENT
        log = self.transport.factory.canaryservice.log
        log(logdata,
            logtype=logtype,
            src_host=peer.address.host,
            src_port=peer.address.port,
            dst_host=us.address.host,
            dst_port=us.address.port
        )

        self.bannerSent = False

    def sendBanner(self):
        if self.bannerSent:
            return
        data = ''
        data = '\r\n'.join(data.splitlines() + [''])
        self.transport.sendPacket(
            userauth.MSG_USERAUTH_BANNER, NS(data) + NS('en'))
        self.bannerSent = True

    def auth_password(self, packet):
        """
        Password authentication.  Payload::
            string password

        Make a UsernamePassword credential and verify it with our portal.
        """
        password = getNS(packet[1:])[0]
        c = credentials.UsernamePassword(self.user, password)

        us = self.transport.getHost()
        peer = self.transport.getPeer()

        logdata = {'USERNAME': self.user, 'PASSWORD': password, 'LOCALVERSION': self.transport.ourVersionString, 'REMOTEVERSION': self.transport.otherVersionString}
        logtype =  self.transport.factory.canaryservice.logger.LOG_SSH_LOGIN_ATTEMPT

        log = self.transport.factory.canaryservice.log
        log(logdata,
            logtype=logtype,
            src_host=peer.address.host,
            src_port=peer.address.port,
            dst_host=us.address.host,
            dst_port=us.address.port)

        return self.portal.login(c, None, conchinterfaces.IConchUser).addErrback(
                                                        self._ebPassword)

    def auth_publickey(self, packet):

        try:
            #extract the public key blob from the SSH packet
            key_blob = getNS(getNS(packet[1:])[1])[0]
        except:
            key_blob = "No public key found."

        try:
            #convert blob into openssh key format
            key = keys.Key.fromString(key_blob).toString('openssh')
        except:
            key = "Invalid SSH Public Key Submitted: {key_blob}".format(key_blob=key_blob.encode('hex'))
            for keytype in ['ecdsa-sha2-nistp256','ecdsa-sha2-nistp384','ecdsa-sha2-nistp521','ssh-ed25519']:
                if keytype in key_blob:
                    key = '{keytype} {keydata}'.format(
                            keytype=keytype,
                            keydata=base64.b64encode(key_blob))

            print 'Key was {key}'.format(key=key)

        c = credentials.SSHPrivateKey(None,None,None,None,None)

        #self.log(key=key)

        return self.portal.login(c, None, conchinterfaces.IConchUser).addErrback(
                                                        self._ebPassword)

    def ssh_USERAUTH_REQUEST(self, packet):
        self.sendBanner()
        return userauth.SSHUserAuthServer.ssh_USERAUTH_REQUEST(self, packet)

# As implemented by Kojoney
class HoneyPotSSHFactory(factory.SSHFactory):
    services = {
        'ssh-userauth': HoneyPotSSHUserAuthServer,
        'ssh-connection': connection.SSHConnection,
        }

    # Special delivery to the loggers to avoid scope problems
    def logDispatch(self, sessionid, msg):
        data = {}
        data['logdata'] = msg
        self.logger.log(data)
        #for dblog in self.dbloggers:
        #    dblog.logDispatch(sessionid, msg)

    def __init__(self, logger=None, version=None):
        # protocol^Wwhatever instances are kept here for the interact feature
        self.sessions = {}
        self.logger = logger
        self.version = version

    def buildProtocol(self, addr):
        # FIXME: try to mimic something real 100%
        t = HoneyPotTransport()
        _modulis = '/etc/ssh/moduli', '/private/etc/moduli'

        if self.version:
            t.ourVersionString = self.version
        else:
            t.ourVersionString = 'empty'

        t.supportedPublicKeys = self.privateKeys.keys()
        for _moduli in _modulis:
            try:
                self.primes = primes.parseModuliFile(_moduli)
                break
            except IOError:
                pass

        if not self.primes:
            ske = t.supportedKeyExchanges[:]
            ske.remove('diffie-hellman-group-exchange-sha1')
            t.supportedKeyExchanges = ske

        t.factory = self
        return t

class HoneyPotRealm:
    implements(portal.IRealm)

    def __init__(self):
        pass

    def requestAvatar(self, avatarId, mind, *interfaces):
        if conchinterfaces.IConchUser in interfaces:
            return interfaces[0], \
                HoneyPotAvatar(avatarId, self.env), lambda: None
        else:
            raise Exception, "No supported interfaces found."

class HoneyPotTransport(transport.SSHServerTransport):

    hadVersion = False

    def connectionMade(self):
        logdata = {'SESSION': str(self.transport.sessionno)}
        logtype = self.factory.canaryservice.logger.LOG_SSH_NEW_CONNECTION
        log = self.factory.canaryservice.log
        log(logdata, transport=self.transport, logtype=logtype)

        self.interactors = []
        self.logintime = time.time()
        self.ttylog_open = False
        transport.SSHServerTransport.connectionMade(self)

    def sendKexInit(self):
        # Don't send key exchange prematurely
        if not self.gotVersion:
            return
        transport.SSHServerTransport.sendKexInit(self)

    def dataReceived(self, data):
        transport.SSHServerTransport.dataReceived(self, data)
        # later versions seem to call sendKexInit again on their own
        isLibssh = data.find('libssh', data.find('SSH-')) != -1

        if (twisted.version.major < 11 or isLibssh) and \
                not self.hadVersion and self.gotVersion:
            self.sendKexInit()
            self.hadVersion = True

    def ssh_KEXINIT(self, packet):
        #print 'Remote SSH version: %s' % (self.otherVersionString,)
        return transport.SSHServerTransport.ssh_KEXINIT(self, packet)

    def ssh_KEX_DH_GEX_REQUEST(self, packet):
        MSG_KEX_DH_GEX_GROUP = 31
        #We have to override this method since the original will
        #pick the client's ideal DH group size. For some SSH clients, this is
        #8192 bits, which takes minutes to compute. Instead, we pick the minimum,
        #which on our test client was 1024.
        if self.ignoreNextPacket:
            self.ignoreNextPacket = 0
            return
        self.dhGexRequest = packet
        min, ideal, max = struct.unpack('>3L', packet)
        self.g, self.p = self.factory.getDHPrime(min)
        self.sendPacket(MSG_KEX_DH_GEX_GROUP, MP(self.p) + MP(self.g))

    def lastlogExit(self):
        starttime = time.strftime('%a %b %d %H:%M',
            time.localtime(self.logintime))
        endtime = time.strftime('%H:%M',
            time.localtime(time.time()))
        duration = str((time.time() - self.logintime))
        clientIP = self.transport.getPeer().host
        #print('root\tpts/0\t%s\t%s - %s (%s)' % \
        #    (clientIP, starttime, endtime, duration))

    # this seems to be the only reliable place of catching lost connection
    def connectionLost(self, reason):
        for i in self.interactors:
            i.sessionClosed()
        if self.transport.sessionno in self.factory.sessions:
            del self.factory.sessions[self.transport.sessionno]
        #self.lastlogExit()
        if self.ttylog_open:
            ttylog.ttylog_close(self.ttylog_file, time.time())
            self.ttylog_open = False
        transport.SSHServerTransport.connectionLost(self, reason)

    def sendDisconnect(self, reason, desc):
        """
        Workaround for the "bad packet length" error message.

        @param reason: the reason for the disconnect.  Should be one of the
                       DISCONNECT_* values.
        @type reason: C{int}
        @param desc: a descrption of the reason for the disconnection.
        @type desc: C{str}
        """
        if not 'bad packet length' in desc:
            # With python >= 3 we can use super?
            transport.SSHServerTransport.sendDisconnect(self, reason, desc)
        else:
            self.transport.write('Protocol mismatch.\n')
            log.msg('Disconnecting with error, code %s\nreason: %s' % \
                (reason, desc))
            self.transport.loseConnection()

class HoneyPotSSHSession(session.SSHSession):
    def request_env(self, data):
        #print 'request_env: %s' % (repr(data))
        pass

class HoneyPotAvatar(avatar.ConchUser):
    implements(conchinterfaces.ISession)

    def __init__(self, username, env):
        avatar.ConchUser.__init__(self)
        self.username = username
        self.env = env
        self.channelLookup.update({'session': HoneyPotSSHSession})

    def openShell(self, protocol):
        return

    def getPty(self, terminal, windowSize, attrs):
        return None

    def execCommand(self, protocol, cmd):
        return

    def closed(self):
        pass

    def eofReceived(self):
        pass

    def windowChanged(self, windowSize):
        self.windowSize = windowSize

def getRSAKeys():
    public_key = os.path.join(SSH_PATH, 'id_rsa.pub')
    private_key = os.path.join(SSH_PATH, 'id_rsa')

    if not (os.path.exists(public_key) and os.path.exists(private_key)):
        from Crypto.PublicKey import RSA
        from twisted.python import randbytes
        KEY_LENGTH = 2048
        rsaKey = RSA.generate(KEY_LENGTH, randbytes.secureRandom)
        publicKeyString = keys.Key(rsaKey).public().toString('openssh')
        privateKeyString = keys.Key(rsaKey).toString('openssh')
        with file(public_key, 'w+b') as f:
            f.write(publicKeyString)
        with file(private_key, 'w+b') as f:
            f.write(privateKeyString)
    else:
        with file(public_key) as f:
            publicKeyString = f.read()
        with file(private_key) as f:
            privateKeyString = f.read()
    return publicKeyString, privateKeyString

def getDSAKeys():
    public_key = os.path.join(SSH_PATH, 'id_dsa.pub')
    private_key = os.path.join(SSH_PATH, 'id_dsa')

    if not (os.path.exists(public_key) and os.path.exists(private_key)):
        from Crypto.PublicKey import DSA
        from twisted.python import randbytes
        KEY_LENGTH = 1024
        dsaKey = DSA.generate(KEY_LENGTH, randbytes.secureRandom)
        publicKeyString = keys.Key(dsaKey).public().toString('openssh')
        privateKeyString = keys.Key(dsaKey).toString('openssh')
        with file(public_key, 'w+b') as f:
            f.write(publicKeyString)
        with file(private_key, 'w+b') as f:
            f.write(privateKeyString)
    else:
        with file(public_key) as f:
            publicKeyString = f.read()
        with file(private_key) as f:
            privateKeyString = f.read()
    return publicKeyString, privateKeyString

class HoneypotPasswordChecker:
    implements(checkers.ICredentialsChecker)

    credentialInterfaces = (credentials.IUsernamePassword,)

    def __init__(self, logger=None):
        self.logger = logger
        self.auth_attempt = 0


    def requestAvatarId(self, credentials):
        return defer.fail(error.UnauthorizedLogin())

class CanaryPublicKeyChecker:
    implements(checkers.ICredentialsChecker)

    credentialInterfaces = (credentials.ISSHPrivateKey,)

    def __init__(self, logger=None):
        self.logger = logger
        self.auth_attempt = 0

    def requestAvatarId(self, credentials):
        return defer.fail(error.UnauthorizedLogin())

class CanarySSH(CanaryService):
    NAME = 'ssh'

    def __init__(self,config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal("ssh.port", default=22))
        self.version = config.getVal("ssh.version", default="SSH-2.0-OpenSSH_5.1p1 Debian-5").encode('utf8')
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        factory = HoneyPotSSHFactory(version=self.version, logger=self.logger)
        factory.canaryservice = self
        factory.portal = portal.Portal(HoneyPotRealm())

        rsa_pubKeyString, rsa_privKeyString = getRSAKeys()
        dsa_pubKeyString, dsa_privKeyString = getDSAKeys()
        factory.portal.registerChecker(HoneypotPasswordChecker(logger=factory.logger))
        factory.portal.registerChecker(CanaryPublicKeyChecker(logger=factory.logger))
        factory.publicKeys = {'ssh-rsa': keys.Key.fromString(data=rsa_pubKeyString),
                              'ssh-dss': keys.Key.fromString(data=dsa_pubKeyString)}
        factory.privateKeys = {'ssh-rsa': keys.Key.fromString(data=rsa_privKeyString),
                               'ssh-dss': keys.Key.fromString(data=dsa_privKeyString)}
        return internet.TCPServer(self.port, factory, interface=self.listen_addr)



from opencanary.modules import CanaryService

from twisted.internet.protocol import Protocol
from twisted.internet.protocol import Factory
from twisted.application import internet

import shlex

class ProtocolError(Exception):
    def __init__(self, reason):
        self.message = '-ERR Protocol error: {reason}\r\n'.format(reason=reason)

class ArgumentCountError(Exception):
    def __init__(self, cmd):
        self.message = "-ERR wrong number of arguments for '{cmd}' command\r\n".format(cmd=cmd.lower())

class AuthenticationRequiredError(Exception):
    def __init__(self):
        self.message = "-NOAUTH Authentication required.\r\n"

class AuthenticationError(Exception):
    def __init__(self):
        self.message = "-ERR invalid password\r\n"

class UnknownCommandError(Exception):
    def __init__(self, cmd):
        cmd.replace('\r',' ').replace('\n', ' ')
        self.message = "-ERR unknown command '{cmd}'\r\n".format(cmd=cmd.lower())

class RedisCommandAgain(Exception):
    pass

class RedisParser:
    pass

class RedisProtocol(Protocol):
    """
        Implementation of basic RESP, that needs authentication.
    """
    COMMANDS = {
        'APPEND': (2,2),
        'AUTH': (1,1),
        'BGREWRITEAOF': (0,0),
        'BGSAVE': (0,None),
        'BITCOUNT': (1,None),
        'BITFIELD': (1,None),
        'BITOP': (3,None),
        'BITPOS': (2,None),
        'BLPOP': (2,None),
        'BRPOP': (2,None),
        'BRPOPLPUSH': (3,3),
        'CLIENT GETNAME': (0,None),
        'CLIENT KILL': (0,None),
        'CLIENT LIST': (0,None),
        'CLIENT PAUSE': (0,None),
        'CLIENT REPLY': (0,None),
        'CLIENT SETNAME': (0,None),
        'CLUSTER ADDSLOTS': (0,None),
        'CLUSTER COUNT-FAILURE-REPORTS': (0,None),
        'CLUSTER COUNTKEYSINSLOT': (0,None),
        'CLUSTER DELSLOTS': (0,None),
        'CLUSTER FAILOVER': (0,None),
        'CLUSTER FORGET': (0,None),
        'CLUSTER GETKEYSINSLOT': (0,None),
        'CLUSTER INFO': (0,None),
        'CLUSTER KEYSLOT': (0,None),
        'CLUSTER MEET': (0,None),
        'CLUSTER NODES': (0,None),
        'CLUSTER REPLICATE': (0,None),
        'CLUSTER RESET': (0,None),
        'CLUSTER SAVECONFIG': (0,None),
        'CLUSTER SET-CONFIG-EPOCH': (0,None),
        'CLUSTER SETSLOT': (0,None),
        'CLUSTER SLAVES': (0,None),
        'CLUSTER SLOTS': (0,None),
        'COMMAND COUNT': (0,None),
        'COMMAND GETKEYS': (0,None),
        'COMMAND INFO': (0,None),
        'COMMAND': (0,None),
        'CONFIG GET': (0,None),
        'CONFIG RESETSTAT': (0,None),
        'CONFIG REWRITE': (0,None),
        'CONFIG SET': (0,None),
        'DBSIZE': (0,0),
        'DEBUG OBJECT': (0,None),
        'DEBUG SEGFAULT': (0,None),
        'DECR': (1,1),
        'DECRBY': (2,2),
        'DEL': (1,None),
        'DISCARD': (0,0),
        'DUMP': (1,1),
        'ECHO': (1,1),
        'EVAL': (2,None),
        'EVALSHA': (2,None),
        'EXEC': (0,0),
        'EXISTS': (1,None),
        'EXPIRE': (2,2),
        'EXPIREAT': (2,2),
        'FLUSHALL': (0,0),
        'FLUSHDB': (0,0),
        'GEOADD': (4,None),
        'GEODIST': (3,None),
        'GEOHASH': (1,None),
        'GEOPOS': (1,None),
        'GEORADIUS': (5,None),
        'GEORADIUSBYMEMBER': (4,None),
        'GET': (1,1),
        'GETBIT': (2,2),
        'GETRANGE': (3,3),
        'GETSET': (2,2),
        'HDEL': (2,None),
        'HEXISTS': (2,2),
        'HGET': (2,2),
        'HGETALL': (1,1),
        'HINCRBY': (3,3),
        'HINCRBYFLOAT': (3,3),
        'HKEYS': (1,1),
        'HLEN': (1,1),
        'HMGET': (2,None),
        'HMSET': (3,None),
        'HSCAN': (2,None),
        'HSET': (3,3),
        'HSETNX': (3,3),
        'HSTRLEN': (2,2),
        'HVALS': (1,1),
        'INCR': (1,1),
        'INCRBY': (2,2),
        'INCRBYFLOAT': (2,2),
        'INFO': (0,None),
        'KEYS': (1,1),
        'LASTSAVE': (0,0),
        'LINDEX': (2,2),
        'LINSERT': (4,4),
        'LLEN': (1,1),
        'LPOP': (1,1),
        'LPUSH': (2,None),
        'LPUSHX': (2,2),
        'LRANGE': (3,3),
        'LREM': (3,3),
        'LSET': (3,3),
        'LTRIM': (3,3),
        'MGET': (1,None),
        'MIGRATE': (5,None),
        'MONITOR': (0,0),
        'MOVE': (2,2),
        'MSET': (2,None),
        'MSETNX': (2,None),
        'MULTI': (0,0),
        'OBJECT': (2,2),
        'PERSIST': (1,1),
        'PEXPIRE': (2,2),
        'PEXPIREAT': (2,2),
        'PFADD': (1,None),
        'PFCOUNT': (1,None),
        'PFMERGE': (1,None),
        'PING': (0,None),
        'PSETEX': (3,3),
        'PSUBSCRIBE': (1,None),
        'PTTL': (1,1),
        'PUBLISH': (2,2),
        'PUBSUB': (1,None),
        'PUNSUBSCRIBE': (0,None),
        'QUIT': (0,None),
        'RANDOMKEY': (0,0),
        'READONLY': (0,0),
        'READWRITE': (0,0),
        'RENAME': (2,2),
        'RENAMENX': (2,2),
        'RESTORE': (3,None),
        'ROLE': (0,0),
        'RPOP': (1,1),
        'RPOPLPUSH': (2,2),
        'RPUSH': (2,None),
        'RPUSHX': (2,2),
        'SADD': (2,None),
        'SAVE': (0,0),
        'SCAN': (1,None),
        'SCARD': (1,1),
        'SCRIPT DEBUG': (0,None),
        'SCRIPT EXISTS': (0,None),
        'SCRIPT FLUSH': (0,None),
        'SCRIPT KILL': (0,None),
        'SCRIPT LOAD': (0,None),
        'SDIFF': (1,None),
        'SDIFFSTORE': (2,None),
        'SELECT': (1,1),
        'SET': (2,None),
        'SETBIT': (3,3),
        'SETEX': (3,3),
        'SETNX': (2,2),
        'SETRANGE': (3,3),
        'SHUTDOWN': (0,None),
        'SINTER': (1,None),
        'SINTERSTORE': (2,None),
        'SISMEMBER': (2,2),
        'SLAVEOF': (2,2),
        'SLOWLOG': (1,None),
        'SMEMBERS': (1,1),
        'SMOVE': (3,3),
        'SORT': (1,None),
        'SPOP': (1,None),
        'SRANDMEMBER': (1,None),
        'SREM': (2,None),
        'SSCAN': (2,None),
        'STRLEN': (1,1),
        'SUBSCRIBE': (1,None),
        'SUNION': (1,None),
        'SUNIONSTORE': (2,None),
        'SYNC': (0,0),
        'TIME': (0,0),
        'TOUCH': (1,None),
        'TTL': (1,1),
        'TYPE': (1,1),
        'UNSUBSCRIBE': (0,None),
        'UNWATCH': (0,0),
        'WAIT': (2,2),
        'WATCH': (1,None),
        'ZADD': (3,None),
        'ZCARD': (1,1),
        'ZCOUNT': (3,3),
        'ZINCRBY': (3,3),
        'ZINTERSTORE': (3,None),
        'ZLEXCOUNT': (3,3),
        'ZRANGE': (3,None),
        'ZRANGEBYLEX': (3,None),
        'ZRANGEBYSCORE': (3,None),
        'ZRANK': (2,2),
        'ZREM': (2,None),
        'ZREMRANGEBYLEX': (3,3),
        'ZREMRANGEBYRANK': (3,3),
        'ZREMRANGEBYSCORE': (3,3),
        'ZREVRANGE': (3,None),
        'ZREVRANGEBYLEX': (3,None),
        'ZREVRANGEBYSCORE': (3,None),
        'ZREVRANK': (2,2),
        'ZSCAN': (2,None),
        'ZSCORE': (2,2),
        'ZUNIONSTORE': (3,None)
    }

    def _buildResponseAndSend(self, input_cmd, input_args):
        try:
            input_cmd = input_cmd.upper()

            if input_cmd not in self.COMMANDS:
                raise UnknownCommandError(input_cmd)

            arg_min_count = self.COMMANDS[input_cmd][0]
            arg_max_count = self.COMMANDS[input_cmd][1]
            input_arg_count = len(input_args)

            if input_arg_count < arg_min_count or \
                    (arg_max_count != None and input_arg_count > arg_max_count):
                raise ArgumentCountError(input_cmd)

            if input_cmd == "QUIT":
                self.transport.write('+OK\r\n')
                self.transport.loseConnection()
                return

            if input_cmd == "AUTH":
                raise AuthenticationError()

            raise AuthenticationRequiredError()

        except (UnknownCommandError, ArgumentCountError, AuthenticationError,
                AuthenticationRequiredError) as e:
            self._logAlert(input_cmd, input_args)
            self.transport.write(e.message)
        return


    def _logAlert(self, cmd, args):
        args = ' '.join(args)
        if len(args) > self.factory.max_arg_length:
            args = args[:self.factory.max_arg_length] + \
                    '(and '+str(self.factory.max_arg_length-len(args))+' more bytes)'
        logdata = {"CMD": cmd,
                   "ARGS": args}
        self.factory.log(logdata, transport=self.transport)

    def _processRedisCommand(self,):
        def _parseInlineCommand(cmd_string):
            try:
                tokens = shlex.split(cmd_string)
                if len(tokens) == 0:
                    cmd = ''
                    args = ''
                else:
                    cmd = tokens[0]
                    args = tokens[1:]
                return cmd, args, ''
            except ValueError:
                raise ProtocolError('unbalanced quotes in request')

        def _parseRESPArray(cmd_string):
            """
                RESP arrays with strings looks like:
                    *<element_count>\r\n
                    $<string_1_length>\r\n
                    <string_1>\r\n
                    $<string_2_length\r\n
                    <string_2>\r\n
            """
            array = []

            if cmd_string[0] != '*':
                raise ProtocolError("expected '*', got '{c}'"
                                        .format(c=cmd_string[0]))

            curr_ptr = cmd_string.find('\r\n')
            if curr_ptr == -1:
                raise RedisCommandAgain()

            arr_count = cmd_string[1:curr_ptr]
            curr_ptr += 2 #skip past CRLF
            try:
                arr_count = int(arr_count)
            except ValueError:
                raise ProtocolError('invalid multibulk length')

            for element_number in range(0,arr_count):
                elem_str, new_ptr = _parseRESPString(cmd_string[curr_ptr:])
                array.append(elem_str)
                curr_ptr += new_ptr

            return array, curr_ptr

        def _parseRESPString(cmd_string):

            if cmd_string[0] != '$':
                raise ProtocolError("expected '$', got '{c}'"
                                        .format(c=cmd_string[0]))

            curr_ptr = cmd_string.find('\r\n')
            if curr_ptr == -1:
                raise RedisCommandAgain()

            str_length = cmd_string[1:curr_ptr]
            curr_ptr += 2 #skip past CRLF
            try:
                str_length = int(str_length)
            except ValueError:
                raise ProtocolError('invalid bulk length')

            resp_str = cmd_string[curr_ptr:(curr_ptr+str_length+2)] #string + CRLF
            if len(resp_str) + 2 < str_length or resp_str[-2:] != '\r\n':
                raise RedisCommandAgain()

            curr_ptr += str_length + 2#string + trailing CRLF
            return resp_str[:-2], curr_ptr

        def _parseRESPCommand(cmd_string):
            cmd = ''
            args = []
            if cmd_string[0] == '*':
                try:
                    array, curr_ptr = _parseRESPArray(cmd_string)
                except IndexError:
                    raise RedisCommandAgain()
            else:
                raise ProtocolError("expected '$', got '{c}'".format(c=cmd_string[0]))
            #print array
            cmd = array[0]
            args = array[1:]
            unfinished_data = cmd_string[curr_ptr:]

            return cmd, args, unfinished_data

        commands = []
        while len(self._data) > 0:
            if self._data[0] == '*':
                #print self._data
                cmd, args, self._data = _parseRESPCommand(self._data)
                #print cmd, args
            else:
                cmd, args, self._data = _parseInlineCommand(self._data)
            commands.append((cmd, args))

        return commands

    def dataReceived(self, data):
        """
        Recieved data is unbuffered so we buffer it for telnet.
        """
        try:
            try:
                #A command split over multiple packets is collected into a single
                #string until it can all be processed. If multiple commands
                #are in a single packet and missing some data, run again

                if not hasattr(self, '_data'):
                    self._data = data
                else:
                    self._data += data

                cmds = self._processRedisCommand()

                for (cmd, args) in cmds:
                    self._buildResponseAndSend(cmd, args)

            except RedisCommandAgain:
                pass

        except (ProtocolError) as e:
            self._errorAndClose(e.message)
            return

    def _errorAndClose(self, error_msg):
        self.transport.write(error_msg+'\r\n')
        self.transport.loseConnection()

class CanaryRedis(Factory, CanaryService):
    NAME = 'redis'
    protocol = RedisProtocol

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = config.getVal("redis.port", default=6379)
        self.max_arg_length = config.getVal("redis.max_arg_length", default=30)
        self.logtype = logger.LOG_REDIS_COMMAND

    def getService(self):
        return internet.TCPServer(self.port, self)



from setuptools import setup, find_packages

import os
import opencanary

setup(
    name='opencanary',
    version=opencanary.__version__,
    url='http://www.thinkst.com/',
    author='Thinkst Applied Research',
    author_email='info@thinkst.com',
    description='OpenCanary daemon',
    long_description='A low interaction honeypot intended to be run on internal networks.',
    install_requires=[
        'Jinja2>=2.4',
        'Twisted==18.4.0',
        'pyasn1==0.4.5',
        'pycrypto==2.6.1',
        'simplejson==3.16.0',
        'wsgiref==0.1.2',
        'requests==2.7.0'
        'zope.interface==4.4.2',
        'PyPDF2==1.26.0',
        'fpdf==1.7.2',
        'passlib==1.7.1',
        'ntlmlib==0.71'
    ],
    setup_requires=[
        'setuptools_git'
    ],
    license='BSD',
    packages = find_packages(exclude='test'),
    scripts=['bin/opencanaryd','bin/opencanary.tac'],
    platforms='any',
    include_package_data=True
)



from opencanary.modules import CanaryService

from zope.interface import implements
from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol

from twisted.application.internet import UDPServer
from twisted.internet.address import IPv4Address

from twisted.internet import protocol

"""
    A log-only NTP server. It won't respond, but it will log attempts
    to trigger the MON_GETLIST_1 NTP commands, which is used for DDOS
    and network recon.
"""

class MiniNtp(DatagramProtocol):
    def datagramReceived(self, data, (host, port)):
        if len(data) < 4 or data[3]!= '\x2a': #monlist command:
            #bogus packet, discard
            return
        logdata={'NTP CMD': 'monlist'}
        self.transport.getPeer = lambda: IPv4Address('UDP', host, port)
        self.factory.log(logdata=logdata, transport=self.transport)

class CanaryNtp(CanaryService):
    NAME = 'ntp'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('ntp.port', default=123))
        self.logtype = logger.LOG_NTP_MONLIST
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        f = MiniNtp()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)



from opencanary.modules import CanaryService

from zope.interface import implements
from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol
from twisted.protocols.sip import Base

from twisted.application.internet import UDPServer
from twisted.internet.address import IPv4Address

from twisted.internet import protocol

"""
    A log-only SIP server. It won't respond, but it will log any
    SIP requests sent its way.
"""

class SIPServer(Base):
    def handle_request(self, request, addr):
        try:
            logdata={'HEADERS': request.headers.data}
            self.transport.getPeer = lambda: IPv4Address('UDP', addr[0], addr[1])
            self.factory.log(logdata=logdata, transport=self.transport)
        except Exception as e:
            self.factory.log(logdata={'ERROR': e}, transport=self.transport)

class CanarySIP(CanaryService):
    NAME = 'SIP'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('sip.port', default=5060))
        self.logtype=self.logger.LOG_SIP_REQUEST
        self.listen_addr = config.getVal('device.listen_addr', default='')


    def getService(self):
        f = SIPServer()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)



class TestPDF:
    FILLER = """
    EMUFPHZLRFAXYUSDJKZLDKRNSHGNFIVJ
    YQTQUXQBQVYUVLLTREVJYQTMKYRDMFD
    VFPJUDEEHZWETZYVGWHKKQETGFQJNCE
    GGWHKK?DQMCPFQZDQMMIAGPFXHQRLG
    TIMVMZJANQLVKQEDAGDVFRPJUNGEUNA
    QZGZLECGYUXUEENJTBJLBQCRTBJDFHRR
    YIZETKZEMVDUFKSJHKFWHKUWQLSZFTI
    HHDDDUVH?DWKBFUFPWNTDFIYCUQZERE
    EVLDKFEZMOQQJLTTUGSYQPFEUNLAVIDX
    FLGGTEZ?FKZBSFDQVGOGIPUFXHHDRKF
    FHQNTGPUAECNUVPDJMQCLQUMUNEDFQ
    ELZZVRRGKFFVOEEXBDMVPNFQXEZLGRE
    DNQFMPNZGLFLPMRJQYALMGNUVPDXVKP
    DQUMEBEDMHDAFMJGZNUPLGEWJLLAETG

    ABCDEFGHIJKLMNOPQRSTUVWXYZABCD
    AKRYPTOSABCDEFGHIJLMNQUVWXZKRYP
    BRYPTOSABCDEFGHIJLMNQUVWXZKRYPT
    CYPTOSABCDEFGHIJLMNQUVWXZKRYPTO
    DPTOSABCDEFGHIJLMNQUVWXZKRYPTOS
    ETOSABCDEFGHIJLMNQUVWXZKRYPTOSA
    FOSABCDEFGHIJLMNQUVWXZKRYPTOSAB
    GSABCDEFGHIJLMNQUVWXZKRYPTOSABC
    HABCDEFGHIJLMNQUVWXZKRYPTOSABCD
    IBCDEFGHIJLMNQUVWXZKRYPTOSABCDE
    JCDEFGHIJLMNQUVWXZKRYPTOSABCDEF
    KDEFGHIJLMNQUVWXZKRYPTOSABCDEFG
    LEFGHIJLMNQUVWXZKRYPTOSABCDEFGH
    MFGHIJLMNQUVWXZKRYPTOSABCDEFGHI

    ENDYAHROHNLSRHEOCPTEOIBIDYSHNAIA
    CHTNREYULDSLLSLLNOHSNOSMRWXMNE
    TPRNGATIHNRARPESLNNELEBLPIIACAE
    WMTWNDITEENRAHCTENEUDRETNHAEOE
    TFOLSEDTIWENHAEIOYTEYQHEENCTAYCR
    EIFTBRSPAMHHEWENATAMATEGYEERLB
    TEEFOASFIOTUETUAEOTOARMAEERTNRTI
    BSEDDNIAAHTTMSTEWPIEROAGRIEWFEB
    AECTDDHILCEIHSITEGOEAOSDDRYDLORIT
    RKLMLEHAGTDHARDPNEOHMGFMFEUHE
    ECDMRIPFEIMEHNLSSTTRTVDOHW?OBKR
    UOXOGHULBSOLIFBBWFLRVQQPRNGKSSO
    TWTQSJQSSEKZZWATJKLUDIAWINFBNYP
    VTTMZFPKWGDKZXTJCDIGKUHUAUEKCAR

    NGHIJLMNQUVWXZKRYPTOSABCDEFGHIJL
    OHIJLMNQUVWXZKRYPTOSABCDEFGHIJL
    PIJLMNQUVWXZKRYPTOSABCDEFGHIJLM
    QJLMNQUVWXZKRYPTOSABCDEFGHIJLMN
    RLMNQUVWXZKRYPTOSABCDEFGHIJLMNQ
    SMNQUVWXZKRYPTOSABCDEFGHIJLMNQU
    TNQUVWXZKRYPTOSABCDEFGHIJLMNQUV
    UQUVWXZKRYPTOSABCDEFGHIJLMNQUVW
    VUVWXZKRYPTOSABCDEFGHIJLMNQUVWX
    WVWXZKRYPTOSABCDEFGHIJLMNQUVWXZ
    XWXZKRYPTOSABCDEFGHIJLMNQUVWXZK
    YXZKRYPTOSABCDEFGHIJLMNQUVWXZKR
    ZZKRYPTOSABCDEFGHIJLMNQUVWXZKRY
    ABCDEFGHIJKLMNOPQRSTUVWXYZABCD
     """

    def __init__(self,):
        self.sharepath = './'

    def createPDF(self, name=None, size='10kb'):
        from PyPDF2 import PdfFileReader, PdfFileWriter
        from fpdf import FPDF
        import os
        import random
        name = os.path.basename(name)
        tmp_name = '/tmp/' + name
        output_name = self.sharepath + '/' + name

        if size == '10kb':
            randlength = random.randint(10000,90000)
        elif size == '100kb':
            randlength = random.randint(100000,900000)
        elif size == '1mb':
            randlength = random.randint(1000000,9000000)

        #create file
        pdf=FPDF()
        pdf.add_page()
        pdf.set_font('Arial','B',8)
        pdf.cell(0,0,os.urandom(randlength))
        pdf.output(tmp_name, "F")

        #encrypt it
        output = PdfFileWriter()
        input1 = PdfFileReader(open(tmp_name, "rb"))
        output.encrypt(user_pwd="ihasapass")
        output.addPage(input1.getPage(0))

        outputStream = file(output_name, "wb")
        output.write(outputStream)
        outputStream.close()


pdf = TestPDF()
pdf.createPDF(name='exciting.pdf', size='1mb')



from opencanary.modules import CanaryService

from zope.interface import implements
from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol

from twisted.application.internet import UDPServer
from twisted.internet.address import IPv4Address

from twisted.internet import protocol

"""
    A log-only Tftp server. It won't respond, but it will log attempts
    to either read or write files.
"""

class Tftp(DatagramProtocol):
    def datagramReceived(self, data, (host, port)):
        if len(data) < 5:
            #bogus packet, discard
            return

        if data[:2] == "\x00\x01":
            opcode="READ"
        elif data[:2] == "\x00\x02":
            opcode="WRITE"
        else:
            #don't log other opcodes
            return

        try:
            (filename, mode, ignore) = data[2:].split("\x00")
        except ValueError:
            return

        logdata={'FILENAME': filename, 'OPCODE': opcode, 'MODE': mode}
        self.transport.getPeer = lambda: IPv4Address('UDP', host, port)
        self.factory.log(logdata=logdata, transport=self.transport)

class CanaryTftp(CanaryService):
    NAME = 'tftp'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('tftp.port', default=69))
        self.logtype=self.logger.LOG_TFTP
        self.listen_addr = config.getVal('device.listen_addr', default='')


    def getService(self):
        f = Tftp()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)
